/*
 *
 * Print or set display type in the CMOS RAM. Relies on /dev/nvram
 * driver being present in the kernel (or module loaded). 
 *
 * Compile: gcc -s -Wall cmosdistype.c -o cmosdistype
 * Usage: cmosdistype [displaytype]
 *
 * Tech info:
 *   0x14 bit 5 & 4 contain the display type.
 *
 *						Paul Gortmaker, 01/2001.
 *
 * This code is released under the GPL, blah blah blah...
 *
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <linux/nvram.h>


/* /dev/nvram excludes 14 registers of RTC */
#define RTC_OFFSET 0x0e

const char *dis_type[] = {
			"EGA/VGA",
			"40x25 CGA",
			"80x25 CGA",
			"MDA (Monochrome)"
};
#define N_TYPES 4

void usage(void) {

	int i;

	fprintf(stderr, "Usage: cmosdistype [displaytype]\n");
	fprintf(stderr, "\tset display type in CMOS\n");
	fprintf(stderr, "\tdisplaytype is one of:\n");
	for (i=0;i<N_TYPES;i++)
		fprintf(stderr, "\t\t%d:\t%s\n", i, dis_type[i]);

	exit(-EINVAL);
}

int main (int argc, char **argv) {

	int fd, i, dis;
	unsigned char data, reg_14, current;

	fd = open("/dev/nvram", O_RDWR);
	if (fd == -1) {
		perror("open - /dev/nvram");
		exit(errno);
	}

	lseek(fd, 0x0e - RTC_OFFSET, SEEK_SET);
	read(fd, &data, 1);
	if (data != 0) {
		printf("CMOS status byte is non-zero - exiting!\n");
		exit(-1);
	}

	lseek(fd, 0x14 - RTC_OFFSET, SEEK_SET);
	read(fd, &reg_14, 1);

	current = (reg_14 & 0x30)>>4;

	printf("Current Display Type: %d (%s).\n", current, dis_type[current]);

	if (argc != 2) exit(0);

	dis = atoi(argv[1]);

	if (dis >= N_TYPES) usage();

	printf("Setting display to type %d (%s).\n", dis, dis_type[dis]);

	data = (reg_14 & ~0x30) | (dis << 4);

	lseek(fd, 0x14 - RTC_OFFSET, SEEK_SET);
	write(fd, &data, 1);

	i = ioctl(fd, NVRAM_SETCKS);
	if (i == -1) {
		perror("ioctl NVRAM_SETCKS - /dev/nvram");
		exit(errno);
	}

	return 0;
}

